import os, sys, pygame, random, math, time
import data, objects
from pygame.locals import *
pygame.init()

#fonts
CUSTOMFONT='Bloody.ttf'
HUGEFONT = pygame.font.Font(data.filepath(CUSTOMFONT), 60)
REGULARFONT = pygame.font.Font(data.filepath(CUSTOMFONT), 40)
TINYFONT = pygame.font.Font(data.filepath(CUSTOMFONT), 20)
#events
SCREEN_UPDATE_EVENT=pygame.USEREVENT+1
DEBUG_MODE=pygame.USEREVENT+2
TOGGLE_AUDIO=pygame.USEREVENT+3
FIRE=pygame.USEREVENT+4
#costants
FPS = 30
DEFAULT_SIZE=800, 600
PAUSESCREEN=-5
SPLASHSCREEN=-4
TITLESCREEN=-3
GAMEINTRO=-2
LEVELLIST=-1
PLANETSIZE=30
#level ID
SUN=0
MERCURY=1
VENUS=2
EARTH=3
MARS=4
JUPITER=5
SATURN=6
URANUS=7
NEPTUNE=8
PLUTO=9
#level status
BRIEFING=0
COUNTDOWN=1
STARTED=2
WIN=3
LOSE=4

class Level():
    def __init__(self, num, screen, myAnims, debugOn, musicOn):
        self.num=num
        self.screen=screen
        self.myAnims=myAnims
        self.debugOn=debugOn
        self.musicOn=musicOn
        self.levelComplete=False
        self.levelFreezed=False
        self.sscenter=-1000,self.screen.get_rect().top
        self.asteroids=[]
        self.status=0
        
        self.layerOrder=[None,objects.Mercury,objects.Venus,objects.Mars,objects.Earth,objects.Moon,objects.Jupiter,objects.Saturn,objects.Uranus,objects.Neptune,objects.Pluto,objects.Asteroid,objects.Seed,objects.Snow,objects.LevelMask,objects.GameText,objects.LevelTimer]
        self.levelObjects=pygame.sprite.LayeredUpdates()
        
        self.backgroundImage=pygame.Surface(self.screen.get_size())
        self.topArea=pygame.Rect(0,0,self.screen.get_width(),self.screen.get_height()/6)
        self.topRow1=self.addObject(objects.GameText('',40,(10+self.topArea.left,10+self.topArea.top)))
        self.topRow2=self.addObject(objects.GameText('',20,(10+self.topArea.left,10+self.topArea.top+50)))
        self.gameArea=pygame.Rect(0,self.screen.get_height()/6,self.screen.get_width(),self.screen.get_height()*4/6)
        #self.gameScreen=pygame.Surface((self.gameArea.size))
        self.bottomArea=pygame.Rect(0,self.screen.get_height()*5/6,self.screen.get_width(),self.screen.get_height()/6)
        self.bottomRow1=self.addObject(objects.GameText('',20,(10+self.bottomArea.left,10+self.bottomArea.top)))
        self.bottomRow2=self.addObject(objects.GameText('',20,(10+self.bottomArea.left,10+self.bottomArea.top+30)))
        self.bottomRow3=self.addObject(objects.GameText('',20,(10+self.bottomArea.left,10+self.bottomArea.top+60)))        
        self.levelTimer=self.addObject(objects.LevelTimer(60,(500+self.topArea.left,10+self.topArea.top)))        
        self.countDownText=self.addObject(objects.GameText('',60,(self.gameArea.center)))        
        self.hideTimer()

        self.nextLevel=self.num
        pygame.event.clear()        
     

    def startMusic(self):
        if not pygame.mixer.music.get_busy() and musicOn:
            pygame.mixer.music.play(-1)           


    def addObjectWORKAROUND(self, newObj):
        layer=0
        self.levelObjects.add(newObj)
        if newObj.__class__ in [objects.GameText,objects.LevelTimer]:
            layer=10
        self.levelObjects.change_layer(newObj, layer)
        return newObj
    
    def addObject(self, newObj):
        spriteLayer=0
        '''
        if newObj.__class__ == objects.LevelMask:
            spriteLayer=7        
        elif newObj.__class__ == objects.Pluto:
            spriteLayer=8 
        elif newObj.__class__ == objects.LevelMask:
            spriteLayer=9           
        elif newObj.__class__ in [objects.GameText,objects.LevelTimer]:
            spriteLayer=10
        self.levelObjects.add(newObj, layer=spriteLayer)
        '''
        try:
            spriteLayer=self.layerOrder.index(newObj.__class__)
        except:
            spriteLayer=0
        self.levelObjects.add(newObj, layer=spriteLayer)
        return newObj    

    def getSeconds(self):
        return self.levelTimer.counter/FPS

    def resetTimer(self):
        self.levelTimer.counter=0

    def showTimer(self):
        self.levelTimer.show=True

    def hideTimer(self):
        self.levelTimer.show=False

    def getImage(self, name):
        return self.myImages[name][0]
    
    def getAnim(self, name):
        return self.myImages[name]

    def eventsManagement(self,event):
        if event.type == QUIT:
            sys.exit(0)
        elif event.type in (KEYDOWN,KEYUP,MOUSEMOTION,MOUSEBUTTONUP,MOUSEBUTTONDOWN):
            self.keysActions(event)
        elif event.type == DEBUG_MODE: #debug on/off
            self.debugOn = not self.debugOn            
        elif event.type == TOGGLE_AUDIO: #audio on/off
            self.musicOn = not self.musicOn
            if self.musicOn:
                pygame.mixer.music.play()
            else:
                pygame.mixer.music.stop()
        if self.levelFreezed==True:
            self.nextLevel=PAUSESCREEN
        return self.nextLevel,self.debugOn,self.musicOn


            
    def customUpdate(self):
        pass
        '''        
        #asteroids collision
        for lo2 in self.asteroids:
            if pygame.sprite.collide_circle(self.opponent,lo2):
                self.asteroids.remove(lo2)
                self.levelObjects.remove(lo2)
        '''
    
    def checkStatus(self):
        pass
    
    def update(self):
        self.draw()
        self.checkStatus()
        self.levelObjects.update()
        self.customUpdate()
        self.generateAsteroids()        
        return self.nextLevel,self.debugOn,self.musicOn
    
    def keysActions(self,keyEvent):
        pass
        
    def draw(self):
        self.screen.fill((0, 0, 0))
        pygame.draw.rect(self.screen,(255,0,0),self.topArea)
        pygame.draw.rect(self.screen,(255,255,0),self.gameArea)
        pygame.draw.rect(self.screen,(255,0,255),self.bottomArea)
        
    def generateAsteroids(self):
        for lo in self.asteroids:
            lo.rect.left=lo.rect.left-lo.speed['left']+lo.speed['right']
            lo.rect.top=lo.rect.top-lo.speed['up']+lo.speed['down']        

        if self.levelTimer.counter%(2*FPS)==0:        
            gap=100
            left=random.randint(-gap,self.screen.get_width()+gap)
            if left<0 or left>self.screen.get_width():
                top=random.randint(-gap,self.screen.get_height()+gap)
            else:
                if random.randint(0,1)==0:
                    top=random.randint(-gap,0)
                else:
                    top=random.randint(self.screen.get_height(),self.screen.get_height()+gap)
            size=random.randrange(50,90,10)
            self.asteroids.append(self.addObject(objects.Asteroid(self,(size,size),(left,top))))        


class Splash(Level):
    def __init__(self, num, screen, myAnims, debugOn, musicOn):
        Level.__init__(self, num, screen, myAnims, debugOn, musicOn)
        self.backgroundImage=myAnims['forked_tail_logo'][0]
        self.screen.blit(self.backgroundImage, (0,0))
        self.listOfFiles=[]
        for f in os.listdir(os.path.normpath(os.path.join('data'))):
            if f[-3:] in ('png','jpg'):
                self.listOfFiles.append(f)
        self.bottomRow3.editText('Loading images...')

    def customUpdate(self):
        self.load_images()

    def keysActions(self,keyEvent):
        if keyEvent.type == KEYDOWN:
            if keyEvent.key == K_ESCAPE:
                sys.exit()            
        
    def drawOLD(self):
        self.screen.blit(self.backgroundImage, (self.screen.get_width()/8,0))     
    
    def draw(self):
        self.levelObjects.clear(self.screen, self.backgroundImage)
        pygame.display.update(self.levelObjects.draw(self.screen))

    def generateAsteroids(self):
        '''overwrites the parent class method'''
        pass

    def load_images(self):
        self.bottomRow1.editText(self.bottomRow2.text)
        self.bottomRow2.editText(self.bottomRow3.text)
        try:
            f=self.listOfFiles.pop()
            print 'Loading '+str(f)
            if f.find('forked_tail_logo')>=0:
                size=self.screen.get_width(),self.screen.get_height()
            elif f.find('title')>=0:
                size=self.screen.get_height(),self.screen.get_height()
            elif f.find('story')>=0:
                size=self.screen.get_width(),self.screen.get_height()
            else:
                size=[None,None]
            name=f[:-4]
            anim,name=name[:name.index('_',name.index('_')+1)+1],name[name.index('_',f.index('_')+1)+1:]
            self.bottomRow3.editText(name)
            self.myAnims[name]=data.load_animation(f, size)
        except IndexError:
            self.bottomRow3.editText('Enjoy the game')            
            self.nextLevel=TITLESCREEN
            self.levelComplete=True



class Title(Level):
    def __init__(self, num, screen, myAnims, debugOn, musicOn):
        Level.__init__(self, num, screen, myAnims, debugOn, musicOn)
        self.backgroundImage=myAnims['title'][0]
        self.selectedColor=pygame.Color('Green')
        self.unselectedColor=pygame.Color('Forest Green')
        self.menuText={}
        self.menuText['menu']=['play','audio '+(not self.musicOn)*'on'+(self.musicOn)*'off','credits','quit']
        self.menuText['credits']=['code:','ilseppia','graphic:','circusblatta','music:','midnightstorm']
        self.menuText['quit']=['quit','cancel']
        self.menuValue=['menu',0]

    def keysActions(self,keyEvent):
        if keyEvent.type == KEYDOWN:
            if keyEvent.key == K_ESCAPE:
                if self.menuValue[0]=='menu':
                    self.menuValue=['quit',0]
                else:    
                    self.menuValue=['menu',0]
            if keyEvent.key == K_RETURN:
                if self.menuValue==['quit',0]:
                    sys.exit()
                elif self.menuValue==['menu',0]:
                    self.nextLevel=GAMEINTRO
                    self.levelComplete=True
                elif self.menuValue==['menu',1]:
                    pygame.event.post(pygame.event.Event(TOGGLE_AUDIO))
                elif self.menuValue[0]=='menu':
                    self.menuValue=[self.menuText[self.menuValue[0]][self.menuValue[1]],0]
                else:
                    self.menuValue=['menu',0]
            if keyEvent.key == K_UP:
                self.menuValue[1]=(self.menuValue[1]-1)%len(self.menuText[self.menuValue[0]])
            if keyEvent.key == K_DOWN:
                self.menuValue[1]=(self.menuValue[1]+1)%len(self.menuText[self.menuValue[0]])
    
    def drawMenu(self):
        self.menuText['menu'][1]='audio '+(not self.musicOn)*'on'+(self.musicOn)*'off'                            
        startingPos=[self.screen.get_height(),self.screen.get_width()-self.screen.get_height()]
        for row in range(len(self.menuText[self.menuValue[0]])):
            if row==self.menuValue[1]:
                rowColor=self.selectedColor
            else:
                rowColor=self.unselectedColor
            menuRow=REGULARFONT.render(self.menuText[self.menuValue[0]][row], True, rowColor)
            self.screen.blit(menuRow, [startingPos[0],startingPos[1]+menuRow.get_height()*row])          
        
    def draw(self):
        self.screen.fill((0, 0, 0)) 
        self.screen.blit(self.backgroundImage, (0,0))      
        self.drawMenu()


class PauseScreen(Level):
    def __init__(self, num, screen, myAnims, debugOn, musicOn):
        Level.__init__(self, num, screen, myAnims, debugOn, musicOn)
        self.backgroundImage=self.myAnims['title'][0]
        self.selectedColor=pygame.Color('Green')
        self.unselectedColor=pygame.Color('Forest Green')
        self.menuText={}
        self.menuText['menu']=['continue','audio '+(not self.musicOn)*'on'+(self.musicOn)*'off','credits','quit game']
        self.menuText['audio']=['on','off']
        self.menuText['credits']=['code:','ilseppia','graphic:','circusblatta','music:','midnightstorm']
        self.menuValue=['menu',0]

    def keysActions(self,keyEvent):
        if keyEvent.type == KEYDOWN:
            if keyEvent.key == K_ESCAPE:
                if self.menuValue[0]=='menu':
                    self.levelComplete=True
                else:    
                    self.menuValue=['menu',0]                
            if keyEvent.key == K_RETURN:
                if self.menuValue==['menu',0]:
                    self.levelComplete=True    
                elif self.menuValue==['menu',1]:
                    pygame.event.post(pygame.event.Event(TOGGLE_AUDIO))                                
                elif self.menuValue==['menu',3]:
                    sys.exit()
                elif self.menuValue[0]=='menu':
                    self.menuValue=[self.menuText[self.menuValue[0]][self.menuValue[1]],0]
            if keyEvent.key == K_UP:
                self.menuValue[1]=(self.menuValue[1]-1)%len(self.menuText[self.menuValue[0]])
            if keyEvent.key == K_DOWN:
                self.menuValue[1]=(self.menuValue[1]+1)%len(self.menuText[self.menuValue[0]])
    
    def drawMenu(self):
        self.menuText['menu'][1]='audio '+(not self.musicOn)*'on'+(self.musicOn)*'off'                            
        startingPos=[self.screen.get_height(),self.screen.get_width()-self.screen.get_height()]
        for row in range(len(self.menuText[self.menuValue[0]])):
            if row==self.menuValue[1]:
                rowColor=self.selectedColor
            else:
                rowColor=self.unselectedColor
            menuRow=REGULARFONT.render(self.menuText[self.menuValue[0]][row], True, rowColor)
            self.screen.blit(menuRow, [startingPos[0],startingPos[1]+menuRow.get_height()*row])          
        
    def draw(self):
        self.screen.fill((0, 0, 0)) 
        self.screen.blit(self.backgroundImage, (0,0))      
        self.drawMenu()


class GameIntro(Level):
    def __init__(self, num, screen, myAnims, debugOn, musicOn):
        Level.__init__(self, num, screen, myAnims, debugOn, musicOn)
        self.story=1
        self.screen.fill(pygame.Color('Black'))
        self.backgroundImage=self.myAnims['story'+str(self.story)][0]
        self.screen.blit(self.backgroundImage, (0,0))

        
    def keysActions(self,keyEvent):
        if keyEvent.type == KEYDOWN:
            if keyEvent.key == K_ESCAPE:
                sys.exit()
            elif keyEvent.key == K_RETURN:
                if self.story<5:
                    self.story+=1
                    self.backgroundImage=self.myAnims['story'+str(self.story)][0]
                    self.screen.blit(self.backgroundImage, (0,0))
                else:
                    self.nextLevel=LEVELLIST
                    self.levelComplete=True   
                    
    def draw(self):
        self.levelObjects.clear(self.screen, self.backgroundImage)
        pygame.display.update(self.levelObjects.draw(self.screen))                                     
                    

class LevelList(Level):
    def __init__(self, num, screen, myAnims, debugOn, musicOn, levelsCompleted, PlutoLives):
        #print levelsCompleted
        Level.__init__(self, num, screen, myAnims, debugOn, musicOn)
        self.backgroundImage=pygame.Surface(self.screen.get_size())
        self.screen.blit(self.backgroundImage, (0,0))
        self.sscenter=-1000,self.screen.get_rect().top
        self.selected=None
        
        self.PlutoLives=PlutoLives
        
        self.topRow1.editText('Solar System')
        self.topRow2.editText('Pluto, you still have '+str(self.PlutoLives)+' lives')
        self.bottomRow1.editText('Dimensions are not real.')
        self.bottomRow2.editText('Please do not plan your interstellar trip on this map.')
        self.bottomRow3.editText('')        
        #self.note=self.addObject(objects.LevelText('\n ',self,1,(100,500)))
        
        '''
        self.planets={}
        names=['mercury','venus']#,'earth','mars','jupiter','saturn','uranus','neptune']
        for p in [MERCURY,VENUS]:#,EARTH,MARS,JUPITER,SATURN,URANUS,NEPTUNE]:
            self.planets[p]=objects.Planet(p,names[p-1],self,)
        for p in self.planets.values():
            self.levelObjects.add(p)
        '''
        
        '''
        Mercury 0.3 4.880 km
        Venus 1 12.104 km
        Earth 1 12.756 km
        Mars 0.5 6.794 km
        Jupiter 11 142.984 km
        Saturn 10 120.536 km
        Uranus 4 51.118 km
        Neptune 4 49.532 km
        Sun 100 1.390.000 km
        Pluto 0.2 2.274 km
        '''
        self.sun=self.addObject(objects.Sun(self,(1000,1000),(-250,self.screen.get_height()/2-300),levelsCompleted[SUN]))
        self.addObject(objects.Mercury(self,(100,100),(75,100),levelsCompleted[MERCURY]))
        self.addObject(objects.Venus(self,(150,150),(100,100),levelsCompleted[VENUS]))
        self.addObject(objects.Earth(self,(150,150),(200,100),levelsCompleted[EARTH]))
        self.addObject(objects.Mars(self,(100,100),(300,100),levelsCompleted[MARS]))
        self.addObject(objects.Jupiter(self,(300,300),(250,100),levelsCompleted[JUPITER]))
        self.addObject(objects.Saturn(self,(400,200),(350,100),levelsCompleted[SATURN]))
        self.addObject(objects.Uranus(self,(200,200),(550,100),levelsCompleted[URANUS]))        
        self.addObject(objects.Neptune(self,(150,300),(650,100),levelsCompleted[NEPTUNE]))
        
        #can't understand why the sun starts with wrong frame
        self.sun.changeAnimation(self.sun.frameDefault)
        
        #self.addObject(objects.PlanetOrbit(300,(-200,300),(255,255,255),1))
        #self.sun=self.addObject(objects.PlanetOrbit(300,(-250,300),(255,255,0)))

        self.t=0.00
        self.t_baseSpeed=0.001


    def customUpdate(self):
        self.t=(self.t+self.t_baseSpeed)%0.5
        #print '###',self.t
        for lo in self.levelObjects:                
            if issubclass(lo.__class__,objects.Planet) and lo.__class__<>objects.Sun:
                if lo==self.selected:
                    lo.orbitSpeed=(lo.orbitSpeed-self.t_baseSpeed)%0.5
                planet_t=-0.25+(self.t+lo.orbitSpeed)%0.5
                #print lo.name,planet_t,lo.orbitRadius*math.sin(planet_t*math.pi)
                lo.rect.left=self.sscenter[0]+lo.orbitRadius*math.cos(planet_t*math.pi)
                lo.rect.top=-(self.sscenter[1]+lo.orbitRadius*math.sin(planet_t*math.pi))
        
        anyoneSelected=False
        for lo in self.levelObjects:                
            if issubclass(lo.__class__,objects.SpaceObject) and lo.__class__<>objects.Sun:
                if lo.rect.collidepoint(pygame.mouse.get_pos()):
                    self.selected=lo
                    self.bottomRow1.editText(lo.name)
                    if issubclass(lo.__class__,objects.Planet):
                        self.bottomRow2.editText((1-lo.levelCompleted)*'NOT '+'Completed')
                    else:
                        self.bottomRow2.editText('')
                    anyoneSelected=True
                    break
        if not anyoneSelected:
            self.bottomRow1.editText('Dimensions are not real.')
            self.bottomRow2.editText('Please do not plan your interstellar trip on this map.')
            self.bottomRow3.editText('')                
            self.selected=None                
                
                
                

    def convertDimensions(self,name):
        width,height=myAnims[name][0].get_size()
        return (width/PLANETSIZE),PLANETSIZE
        

        
    def keysActions(self,keyEvent):
        if keyEvent.type == KEYDOWN:
            if keyEvent.key == K_ESCAPE:
                self.levelFreezed=True

        if keyEvent.type == MOUSEBUTTONDOWN:
            if keyEvent.button == 1:
                for lo in self.levelObjects:
                    if issubclass(lo.__class__,objects.Planet) and lo.__class__<>objects.Sun:         
                        if lo.rect.collidepoint(pygame.mouse.get_pos()):
                            try:
                                self.nextLevel=lo.ID
                            except:#remove this try block
                                self.nextLevel=MERCURY
                            self.levelComplete=True
                            break

    def draw(self):
        self.levelObjects.clear(self.screen, self.backgroundImage)
        pygame.display.update((self.levelObjects.draw(self.screen)))



class GameLevel(Level):
    def __init__(self, num, screen, myAnims, debugOn, musicOn):
        Level.__init__(self, num, screen, myAnims, debugOn, musicOn)
        self.backgroundImage=pygame.Surface((self.screen.get_size()))
        self.screen.blit(self.backgroundImage, (0,0))
      
        self.status=BRIEFING

    def checkStatus(self):
        if self.status in (WIN,LOSE):
            pass
        elif self.status==STARTED:
            self.checkWin()
            self.checkLose()
        elif self.status==COUNTDOWN:
            self.countDownText.editText(3-self.getSeconds())
            if self.getSeconds()>=3:
                self.status=STARTED
                self.resetTimer()
                self.showTimer()
                self.countDownText.editText('')
                self.doOnceAtGameStart()
        elif self.status==BRIEFING:
            pass

    def doOnceAtGameStart(self):
        pass

    def customUpdate(self):
        pass
        
    def checkWin(self):
        if False:
            self.levelWin()

    def checkLose(self):
        if False:
            self.levelLose()

    def levelWin(self,msg=''):
        self.you.win=True
        self.you.frameDefault=self.you.frameWin
        self.opponent.lose=True
        self.opponent.frameDefault=self.opponent.frameLose 
        self.status=WIN
        self.bottomRow1.editText(msg)
        self.bottomRow2.editText('I admit you are worthy of being a planet.')
        self.bottomRow3.editText('Press ENTER to continue.')   

    def levelLose(self,msg=''):
        self.you.lose=True
        self.you.frameDefault=self.you.frameLose
        self.opponent.win=True
        self.opponent.frameDefault=self.opponent.frameWin
        self.status=LOSE
        self.bottomRow1.editText(msg)
        self.bottomRow2.editText('Loser! You\'ll never be a real planet!')
        self.bottomRow3.editText('Press ENTER to continue.')   

    
    def keysActions(self,keyEvent):
        if keyEvent.type == KEYDOWN:
            if keyEvent.key == K_ESCAPE:
                self.levelFreezed=True
            elif keyEvent.key == K_RETURN:
                if self.status==BRIEFING:
                    self.status=COUNTDOWN
                    self.resetTimer()
                elif self.status in (WIN,LOSE):
                    self.levelComplete=True
                    self.nextLevel=LEVELLIST
        if self.status==STARTED:                    
            self.customKeysActions(keyEvent)
    
    def customKeysActions(self,keyEvent):
        pass




    def draw(self):
        #pygame.draw.rect(self.screen,(0,0,0),self.topArea)
        #pygame.draw.rect(self.screen,(0,0,0),self.bottomArea)
        self.levelObjects.clear(self.screen, self.backgroundImage)
        pygame.display.update(self.levelObjects.draw(self.screen))#+[self.topArea,self.bottomArea])




class MercuryLevel(GameLevel):
    def __init__(self, num, screen, myAnims, debugOn, musicOn):
        GameLevel.__init__(self, num, screen, myAnims, debugOn, musicOn)
        self.topRow1.editText('Mercury Speed Test')
        self.topRow2.editText('')
        self.bottomRow1.editText('Mercury is currently the "1000 light-years" champion. Can you beat him?')
        self.bottomRow2.editText('Press Z and M keys alternately ro run.')
        self.bottomRow3.editText('Press ENTER when you are ready.')
        
        self.you=self.addObject(objects.Pluto(self,(200,200),(100,100)))
        self.opponent=self.addObject(objects.Mercury(self,(200,200),(100,300)))
        self.signals=[]
        for i in range(1,11):
            self.signals.append(self.addObject(objects.GameText(str(i*100)+' light-years',20,(i*500,300))))
        self.lastPressed=None

    def doOnceAtGameStart(self):
        self.bottomRow1.editText('')
        self.bottomRow2.editText('')
        self.bottomRow3.editText('Press Z and M keys alternately ro run.')
            

    def checkWin(self):
        if self.you.rect.left>self.signals[len(self.signals)-1].rect.left:
            self.levelWin('You win Pluto, you are faster than me.')
            self.opponent.rect.left=self.you.rect.left-50
    
    def checkLose(self):
        if self.opponent.rect.left>self.signals[len(self.signals)-1].rect.left:
            self.levelLose()

            
    def customUpdate(self):
        if self.status==STARTED:
            #increase opponent speed
            self.opponent.speed['left']+=0.04
            #center screen on you
            self.opponent.rect.left=self.opponent.rect.left-self.you.speed['left']+self.opponent.speed['left']
            #move text signals
            for s in self.signals:
                s.rect.left=s.rect.left-self.you.speed['left']


    def customKeysActions(self,keyEvent):
        if keyEvent.type == KEYDOWN:
            if keyEvent.key in (K_z, K_m):
                if self.status==STARTED:
                    if self.lastPressed<>keyEvent.key:
                        self.you.speed['left']+=0.1
                    self.lastPressed=keyEvent.key


class VenusLevel(GameLevel):
    def __init__(self, num, screen, myAnims, debugOn, musicOn):
        GameLevel.__init__(self, num, screen, myAnims, debugOn, musicOn)
        self.topRow1.editText('Venus Attraction')
        self.topRow2.editText('')
        self.bottomRow1.editText('Venus is so charming, asteroids are attracted to her.')
        self.bottomRow2.editText('Show Venus you can take more asteroids than her.')
        self.bottomRow3.editText('Move with ARROW keys. Press ENTER when you are ready.')
        
        self.you=self.addObject(objects.Pluto(self,(200,200),(100,200)))
        self.opponent=self.addObject(objects.Venus(self,(200,200),(300,200)))


    def checkWin(self):
        if self.you.mass>self.opponent.mass:
            self.levelWin('You win Pluto, you are delightfu.')
    
    def checkLose(self):
        if self.getSeconds()>60:
            self.levelLose('Time out!')
           

    def doOnceAtGameStart(self):
        self.bottomRow1.editText('')
        self.bottomRow2.editText('')
        self.bottomRow3.editText('Move with ARROW keys and increase your mass taking the asteroids.')
            
    
            
    def customUpdate(self):
        if self.status==STARTED:
            self.bottomRow1.editText('Venus '+str(self.opponent.mass)+' kg 10E22 - Pluto '+str(self.you.mass)+' kg 10E22')
            for lo in self.levelObjects:
                if issubclass(lo.__class__,objects.Planet):
                    lo.rect.left=lo.rect.left-lo.speed['left']+lo.speed['right']
                    lo.rect.top=lo.rect.top-lo.speed['up']+lo.speed['down']
                    
                if lo in [self.you,self.opponent]:
                    for lo2 in self.levelObjects:
                        if lo2.__class__==objects.Asteroid:
                            if pygame.sprite.collide_circle(lo,lo2):
                                lo.mass+=lo2.mass
                                lo.changeAnimation(lo.frameCollect,1)
                                self.asteroids.remove(lo2)
                                self.levelObjects.remove(lo2)
                    


    

        

        
    def customKeysActions(self,keyEvent):
        if keyEvent.type in (KEYDOWN,KEYUP):
            if keyEvent.type == KEYDOWN:
                pressed=1
            elif keyEvent.type == KEYUP:
                pressed=0
            if keyEvent.key == K_LEFT:
                self.you.engine['left']=pressed
            elif keyEvent.key == K_RIGHT:
                self.you.engine['right']=pressed
            elif keyEvent.key == K_UP:
                self.you.engine['up']=pressed
            elif keyEvent.key == K_DOWN:
                self.you.engine['down']=pressed               



class EarthLevel(GameLevel):
    def __init__(self, num, screen, myAnims, debugOn, musicOn):
        GameLevel.__init__(self, num, screen, myAnims, debugOn, musicOn)
        self.topRow1.editText('Rich soil Earth')
        self.topRow2.editText('')
        self.bottomRow1.editText('Can you be more green than Earth?')
        self.bottomRow2.editText('Drag wandering seeds and asteroids with the mouse.')
        self.bottomRow3.editText('Press ENTER when you are ready.')
        
        self.you=self.addObject(objects.Pluto(self,(220,220),(50,200)))
        self.opponent=self.addObject(objects.Earth(self,(220,220),(500,200)))
        self.moon=self.addObject(objects.Moon(self,(120,120),(650,200)))
        self.easterEggFlag=False
        
        self.seeds=[]
        self.taken=None
        
        self.sscenter=self.screen.get_rect().center
        self.t=0.00
        self.t_baseSpeed=0.01
        self.t_moonSpeed=0.005



    def doOnceAtGameStart(self):
        self.bottomRow1.editText('')
        self.bottomRow2.editText('Hint: Drag seeds on your surface, asteroids on Earth\'s one.')
        self.bottomRow3.editText('You have only one minute. You target is 90% green on Pluto, 10% on Earth.')
        for lo2 in [self.you,self.opponent]:
            nextFrame=range(lo2.frameGreenery[0],lo2.frameGreenery[1]+1)[min(99,lo2.greenery)/20]
            lo2.changeAnimation((nextFrame,nextFrame))
            lo2.frameDefault=(nextFrame,nextFrame)
            
    def checkWin(self):
        if self.you.greenery>=90 and self.opponent.greenery<=10:
            self.levelWin('You win Pluto, you are so fertile!')
    
    def checkLose(self):
        if self.getSeconds()>60:
            self.levelLose('Time out!')
        elif self.you.greenery==0:
            self.levelLose('You are only barren land!')
        elif self.opponent.greenery==100:
            self.levelLose('I am too green for you...')
           

            
    def customUpdate(self):
        if self.status==STARTED:
            self.t=(self.t-self.t_baseSpeed)%2
            self.tmoon=(self.t-self.t_moonSpeed)%2

            self.you.rect.centerx=self.sscenter[0]+200*math.cos(self.t*math.pi)
            self.you.rect.centery=self.sscenter[1]+200*math.sin(self.t*math.pi)
            self.opponent.rect.centerx=self.sscenter[0]+200*math.cos((self.t+1)*math.pi)
            self.opponent.rect.centery=self.sscenter[1]+200*math.sin((self.t+1)*math.pi)
            self.moon.rect.centerx=self.opponent.rect.centerx+130*math.cos(self.tmoon*math.pi)
            self.moon.rect.centery=self.opponent.rect.centery+130*math.sin(self.tmoon*math.pi)

            self.generateSeeds()
            self.topRow2.editText('Greenery - Earth: '+str(self.opponent.greenery)+'% - Pluto: '+str(self.you.greenery)+'%')
            for lo in self.asteroids+self.seeds:
                if lo==self.taken:
                    lo.rect.center=pygame.mouse.get_pos()

            for lo1 in self.asteroids+self.seeds:                    
                for lo2 in [self.you,self.opponent,self.moon]:
                    if pygame.sprite.collide_circle(lo1,lo2):
                        lo2.greenery=max(0,min(100,lo2.greenery+lo1.greenery))
                        if lo2 in [self.you,self.opponent]:
                            nextFrame=range(lo2.frameGreenery[0],lo2.frameGreenery[1]+1)[min(99,lo2.greenery)/20]
                            lo2.changeAnimation((nextFrame,nextFrame))
                            lo2.frameDefault=(nextFrame,nextFrame)
                        try:
                            self.asteroids.remove(lo1)
                        except:
                            pass
                        try:
                            self.seeds.remove(lo1)
                        except:
                            pass
                        self.levelObjects.remove(lo1)
                        self.taken=None
                    

    
       
    def customKeysActions(self,keyEvent):
        if keyEvent.type == MOUSEMOTION:
            if self.taken==None:
                for lo in self.asteroids+self.seeds:
                    if lo.rect.collidepoint(pygame.mouse.get_pos()):
                        self.taken=lo
                        break  
        elif keyEvent.type == MOUSEBUTTONDOWN:
            if keyEvent.button == 1:
                if self.moon.rect.collidepoint(pygame.mouse.get_pos()):
                    if not self.easterEggFlag:
                        self.moon.changeAnimation(self.moon.frameFold,1)
                        self.moon.frameDefault=self.moon.frameNoSign
                        self.easterEggFlag=True
                    else:
                        self.moon.changeAnimation(self.moon.frameUnfold,1)
                        self.moon.frameDefault=self.moon.frameEasterEgg           
                        
                    
  
    def generateSeeds(self):
        for lo in self.seeds:
            lo.rect.left=lo.rect.left-lo.speed['left']+lo.speed['right']
            lo.rect.top=lo.rect.top-lo.speed['up']+lo.speed['down']        

        if self.levelTimer.counter%(2*FPS)==0:        
            gap=100
            left=random.randint(-gap,self.screen.get_width()+gap)
            if left<0 or left>self.screen.get_width():
                top=random.randint(-gap,self.screen.get_height()+gap)
            else:
                if random.randint(0,1)==0:
                    top=random.randint(-gap,0)
                else:
                    top=random.randint(self.screen.get_height(),self.screen.get_height()+gap)
            size=random.randrange(50,90,10)
            self.seeds.append(self.addObject(objects.Seed(self,(size,size),(left,top))))  



class MarsLevel(GameLevel):
    def __init__(self, num, screen, myAnims, debugOn, musicOn):
        GameLevel.__init__(self, num, screen, myAnims, debugOn, musicOn)
        self.topRow1.editText('Mars Figth')
        self.topRow2.editText('')
        self.bottomRow1.editText('')
        self.bottomRow2.editText('Press suggested key combination to attack. Hit Mars 3 times.')
        self.bottomRow3.editText('Press ENTER when you are ready.')
        
        self.you=self.addObject(objects.Pluto(self,(200,200),(500,300)))
        self.opponent=self.addObject(objects.Mars(self,(200,200),(100,300)))
        self.randLetter=[]
        self.consecutiveHits=0
        self.secondsWithoutPress=0
        self.keyHasBeenPressed=True


    def checkWin(self):
        if self.opponent.lives<=0:
            self.levelWin('You win Pluto, you are a real fighter!')
    
    def checkLose(self):
        if self.you.lives<=0:
            self.levelLose('I taught you a lesson!')
            
                

    def doOnceAtGameStart(self):
        self.bottomRow1.editText('')
        self.bottomRow2.editText('')
        self.bottomRow3.editText('Press suggested key combination to attack. Hit Mars 3 times.')
            
    
    def customUpdate(self):
        self.topRow2.editText('LIVES: Mars='+str(self.opponent.lives)+' - Pluto:'+str(self.you.lives))
        if self.status==STARTED:
            if self.levelTimer.counter%(1*FPS)==0:
                self.randLetter.append(random.randint(ord('a'),ord('z')))
                if self.keyHasBeenPressed==True:
                    self.secondsWithoutPress=0
                    self.keyHasBeenPressed=False
                else:
                    self.secondsWithoutPress+=1
            letters=''
            for rl in self.randLetter:
                letters=letters+' '+chr(rl)
            self.bottomRow1.editText(letters)
            
            if self.consecutiveHits>=1:
                self.bottomRow2.editText(str(self.consecutiveHits)+' hits combo!')
                
            if self.secondsWithoutPress>=2:
                self.randLetter.pop(0)
                self.secondsWithoutPress=0
                self.youFailMarsAttack('TOO LATE')                
                
                
            
            
    def customKeysActions(self,keyEvent):
        if keyEvent.type == KEYDOWN:
            self.keyHasBeenPressed=True
            try:
                if pygame.key.name(keyEvent.key)==chr(self.randLetter.pop(0)):
                    self.consecutiveHits+=1
                    if self.consecutiveHits%5==0:
                        self.youHit('TAKE THIS!')
                else:
                    self.youFailMarsAttack('WRONG KEY')                     
            except IndexError:
                self.youFailMarsAttack('WRONG KEY')       


    def youHit(self,msg):
        self.bottomRow2.editText(msg)
        self.randLetter=[]
        self.you.changeAnimation(self.you.frameAttack,1)
        if self.opponent.lives>0:
            self.opponent.lives-=1
            self.opponent.changeAnimation(self.opponent.frameHit,1)
        else:
            self.opponent.changeAnimation(self.opponent.frameLose,1)        
        
                
    def youFailMarsAttack(self,msg):
        self.bottomRow2.editText('FAIL ('+msg+')')
        self.you.changeAnimation(self.you.frameAttackFail,1)
        self.consecutiveHits=0
        self.randLetter=[]
        self.opponent.changeAnimation(self.opponent.frameAttack,1)
        self.you.lives-=1

                
class JupiterLevel(GameLevel):
    def __init__(self, num, screen, myAnims, debugOn, musicOn):
        GameLevel.__init__(self, num, screen, myAnims, debugOn, musicOn)
        self.topRow1.editText('Big Jupiter')
        self.topRow2.editText('')
        self.bottomRow1.editText('Can you swell more than Jupiter?')
        self.bottomRow2.editText('Keep pressed left mouse button to blow you up, but try to not explode.')
        self.bottomRow3.editText('Press ENTER when you are ready.')
        
        self.you=self.addObject(objects.Pluto(self,(200,200),(500,200)))
        self.opponent=self.addObject(objects.Jupiter(self,(300,300),(100,200)))
        self.blowTime=False
        self.checkTime=False
        self.counterCheckTime=0


    def checkWin(self):
        if self.opponent.lives<=0:
            self.levelWin('You win... but you are a cheater!!')
    
    def checkLose(self):
        if self.you.lives<=0:
            self.levelLose('Poor small dwarw planet! Ah ah ah!')
            
                

    def doOnceAtGameStart(self):
        self.bottomRow1.editText('')
        self.bottomRow2.editText('')
        self.bottomRow3.editText('Keep pressed left mouse button to blow you up, but try to not explode.')
        self.blowTime=True
            
    
    def customUpdate(self):
        if self.status==STARTED:
            if self.checkTime:
                if self.levelTimer.counter%(1*FPS)==0:
                    self.counterCheckTime+=1
                if self.counterCheckTime>=5:
                    self.you.changeAnimation(self.you.frameExplode,1)
                    self.you.lives=-1
            
            
    def customKeysActions(self,keyEvent):
        if keyEvent.type == MOUSEBUTTONDOWN:
            if keyEvent.button == 1:
                if self.blowTime:
                    if self.you.rect.collidepoint(pygame.mouse.get_pos()):
                        self.you.changeAnimation(self.you.frameBlow)
                        self.opponent.changeAnimation(self.opponent.frameBlow2)
                elif self.checkTime:
                    if self.opponent.rect.collidepoint(pygame.mouse.get_pos()):
                        self.opponent.changeAnimation(self.opponent.frameDeflate,1)
                        self.opponent.lives-=1
        elif keyEvent.type == MOUSEBUTTONUP:
            if keyEvent.button == 1:
                if self.you.rect.collidepoint(pygame.mouse.get_pos()):
                    if self.you.frameNbr in self.you.frameBlow:
                        self.you.changeAnimation(self.you.frameKeep)
                        self.opponent.changeAnimation(self.opponent.frameKeep2)
                        self.blowTime=False
                        self.checkTime=True



 
class SaturnLevel(GameLevel):
    def __init__(self, num, screen, myAnims, debugOn, musicOn):
        GameLevel.__init__(self, num, screen, myAnims, debugOn, musicOn)
        self.topRow1.editText('Saturn')
        self.topRow2.editText('')
        self.msg=[]
        self.msg.append('Please find my RING!')
        self.msg.append('Where my RING is?')
        self.msg.append('I know you stole my RING!')
        self.msg.append('Look over there...')
        self.msg.append('You are so dumb you can\'t find a RING?')
        self.msg.append('If the phones RING, does the RING phone?')
        self.msg.append('Give me an R! Give me an I! Give me an N! Give me a G!')
        self.msg.append('How can I box without the RING?')
        self.msg.append('Can you ear that noise? It makes my ears RING')
        self.msg.append('That\'s was funny, wasn\'t it?')        
        
        self.bottomRow1.editText('')
        self.bottomRow2.editText('Please find my RING!')
        self.bottomRow3.editText('Press ENTER when you are ready.')
        
        self.you=self.addObject(objects.Pluto(self,(200,200),(100,200)))
        self.opponent=self.addObject(objects.Saturn(self,(400,200),(300,200)))
        self.opponent.changeAnimation(self.opponent.frameWaiting)
        self.ring=['r','i','n','g']
        self.ringCheck=0


    def checkWin(self):
        if self.ringCheck==4:
            self.levelWin('Thank you Pluto, I\'ll give you a RING sooner or later.')
    
    def checkLose(self):
        if self.getSeconds()>60:
            self.levelLose('Looser... you didn\'t find my RING')
           

    def doOnceAtGameStart(self):
        self.bottomRow1.editText('')
        self.bottomRow2.editText('')
        self.bottomRow3.editText('')
            
    
            
    def customUpdate(self):
        if self.status==STARTED:
            if self.levelTimer.counter%(5*FPS)==0:
                self.bottomRow1.editText(self.bottomRow3.text)
                #self.bottomRow2.editText(self.bottomRow3.text)
                self.bottomRow3.editText(self.msg[random.randrange(0,len(self.msg))])                
                
        
    def customKeysActions(self,keyEvent):
        if keyEvent.type in (KEYDOWN,KEYUP):
            if keyEvent.type == KEYDOWN:
                pressed=1
            elif keyEvent.type == KEYUP:
                pressed=0
            if keyEvent.key == K_LEFT:
                self.you.engine['left']=pressed
            elif keyEvent.key == K_RIGHT:
                self.you.engine['right']=pressed
            elif keyEvent.key == K_UP:
                self.you.engine['up']=pressed
            elif keyEvent.key == K_DOWN:
                self.you.engine['down']=pressed
        
        if keyEvent.type==KEYDOWN:
            if pygame.key.name(keyEvent.key)==self.ring[self.ringCheck]:
                self.ringCheck+=1
            else:
                self.ringCheck=0
            


class UranusLevel(GameLevel):
    def __init__(self, num, screen, myAnims, debugOn, musicOn):
        GameLevel.__init__(self, num, screen, myAnims, debugOn, musicOn)
        self.topRow1.editText('Dark Uranus')
        self.topRow2.editText('')
        self.bottomRow1.editText('Where is Urano? He is so dark...')
        self.bottomRow2.editText('Move your mouse to find it. Left click to take a picture.')
        self.bottomRow3.editText('Press ENTER when you are ready.')
        
        self.you=self.addObject(objects.Pluto(self,(200,200),self.screen.get_rect().center))
        #self.opponent=None#self.addObject(objects.Uranus(self,(200,200),(random.randint(100,self.screen.get_width()-100),random.randint(100,self.screen.get_height()-100))))
        self.hiding=False
        
        self.maskGap=50
        self.maskTopLeft=self.addObject(objects.LevelMask((self.you.rect.right+2*self.maskGap,self.you.rect.top+2*self.maskGap),(0-self.maskGap,0-self.maskGap),pygame.Color('Black')))
        self.maskTopRight=self.addObject(objects.LevelMask((self.screen.get_width()-self.you.rect.right+2*self.maskGap,self.you.rect.bottom+2*self.maskGap),(self.you.rect.right-self.maskGap,0-self.maskGap),pygame.Color('Black')))
        self.maskBottomLeft=self.addObject(objects.LevelMask((self.you.rect.left+2*self.maskGap,self.screen.get_height()-self.you.rect.top+2*self.maskGap),(0-self.maskGap,self.you.rect.top-self.maskGap),pygame.Color('Black')))
        self.maskBottomRight=self.addObject(objects.LevelMask((self.screen.get_width()-self.you.rect.left+2*self.maskGap,self.screen.get_height()-self.you.rect.bottom+2*self.maskGap),(self.you.rect.left-self.maskGap,self.you.rect.bottom-self.maskGap),pygame.Color('Black')))
        self.maskCenter=self.addObject(objects.LevelMask(self.you.rect.size,self.you.rect.topleft,pygame.Color('Black')))
        


    def doOnceAtGameStart(self):
        self.you.rect.center=self.screen.get_rect().center
        self.levelObjects.change_layer(self.you,self.layerOrder.index(objects.Pluto))         
        self.maskCenter.rect.topleft=(self.you.rect.topleft)
        self.levelObjects.change_layer(self.maskCenter,self.layerOrder.index(objects.LevelMask))
        self.bottomRow1.editText('')
        self.bottomRow2.editText('')
        
        if not hasattr(self,'opponent'):
            self.bottomRow3.editText('Take 3 pictures of Urano.')
            self.opponent=self.addObject(objects.Uranus(self,(200,200),(random.randint(100,self.screen.get_width()-100),random.randint(100,self.screen.get_height()-100))))             
        else:
            self.bottomRow3.editText('Take '+str(self.opponent.lives)+' more pictures of Urano.')
            self.opponent.rect.left=random.randint(100,self.screen.get_width()-100)
            self.opponent.rect.top=random.randint(100,self.screen.get_height()-100)            
        self.levelObjects.change_layer(self.maskCenter,0)
        self.hiding=False

    def checkWin(self):
        if self.opponent.lives<=0:
            self.you.rect.center=self.screen.get_rect().center
            self.opponent.frameDefault=self.opponent.frameFoundAgain
            self.levelWin('You win Pluto, this was my last hideout!')
    
    def checkLose(self):
        if self.getSeconds()>60:
            self.levelLose('Time out!')
            
                

            
    def customUpdate(self):
        pass
                    
       
    def customKeysActions(self,keyEvent):
        if not self.hiding:
            if keyEvent.type == MOUSEMOTION:
                self.you.rect.center=pygame.mouse.get_pos()
                #self.maskTopLeft.rect.topleft=(0-self.maskGap,0-self.maskGap) #neverchanges
                self.maskTopLeft.rect.size=(max(0,self.you.rect.right+2*self.maskGap),max(0,self.you.rect.top+2*self.maskGap))
                self.maskTopRight.rect.topleft=(self.you.rect.right-self.maskGap,0-self.maskGap)
                self.maskTopRight.rect.size=(max(0,self.screen.get_width()-self.you.rect.right+2*self.maskGap),max(0,self.you.rect.bottom+2*self.maskGap))
                self.maskBottomLeft.rect.topleft=(0-self.maskGap,self.you.rect.top-self.maskGap)
                self.maskBottomLeft.rect.size=(max(0,self.you.rect.left+2*self.maskGap),max(0,self.screen.get_height()-self.you.rect.top+2*self.maskGap))                
                self.maskBottomRight.rect.topleft=(self.you.rect.left-self.maskGap,self.you.rect.bottom-self.maskGap)
                self.maskBottomRight.rect.size=(max(0,self.screen.get_width()-self.you.rect.left+2*self.maskGap),max(0,self.screen.get_height()-self.you.rect.bottom+2*self.maskGap))                
                self.maskCenter.rect.topleft=(self.you.rect.topleft)
            
            elif keyEvent.type == MOUSEBUTTONDOWN:
                if keyEvent.button == 1:
                    if pygame.sprite.collide_circle(self.opponent,self.you):
                        self.youHit('Here you are!')
        
        if self.hiding:            
            if keyEvent.type == KEYDOWN:
                if keyEvent.key == K_RETURN:
                    self.doOnceAtGameStart()

    def youHit(self,msg):
        self.levelObjects.change_layer(self.you,0)  
        self.bottomRow2.editText(msg)
        self.hiding=True
        if self.opponent.lives>0:
            self.opponent.lives-=1
            self.you.changeAnimation(self.you.frameFound,1)
            self.opponent.changeAnimation(self.opponent.frameFound,1)
            self.bottomRow3.editText('Next time it will be harder. Press ENTER to find me again')
        else:
            self.opponent.changeAnimation(self.opponent.frameFoundAgain,1)

                
class NeptuneLevel(GameLevel):
    def __init__(self, num, screen, myAnims, debugOn, musicOn):
        GameLevel.__init__(self, num, screen, myAnims, debugOn, musicOn)
        self.topRow1.editText('Freezing Neptune')
        self.topRow2.editText('')
        self.bottomRow1.editText('Neptune is cold.')
        self.bottomRow2.editText('Collect snow and move mouse to build a snow ball. ')
        self.bottomRow3.editText('Press ENTER when you are ready.')
        
        self.you=self.addObject(objects.Pluto(self,(200,200),(600,200)))
        self.opponent=self.addObject(objects.Neptune(self,(150,300),(50,200)))

        self.snow=[]
        self.taken=None
        self.takenCount=0


    def doOnceAtGameStart(self):
        self.bottomRow1.editText('')
        self.bottomRow2.editText('')
        self.bottomRow3.editText('Collect snow and move mouse to build a snow ball. Then drag it on you.')



    def checkWin(self):
        if self.opponent.lives<=0:
            self.levelWin('You win Pluto, I have flue!')
    
    def checkLose(self):
        if self.getSeconds()>60:
            self.levelLose('Time out!')

    def customUpdate(self):
        if self.status==STARTED:
            self.generateSnow()
            
            for lo in self.snow:
                if lo==self.taken:
                    lo.rect.center=pygame.mouse.get_pos()
                else:
                    lo.rect.left=lo.rect.left-lo.speed['left']+lo.speed['right']
                    lo.rect.top=lo.rect.top-lo.speed['up']+lo.speed['down']
                    
            for lo2 in self.snow:
                if pygame.sprite.collide_circle(self.you,lo2):
                    if lo2.hold and lo2.currentFrame==lo.snowFrame:
                        self.shot()
                        self.taken=None
                        self.takenCount=0
                    self.snow.remove(lo2)
                    self.levelObjects.remove(lo2)
                for lo1 in [self.opponent]+self.asteroids:
                    if pygame.sprite.collide_circle(lo1,lo2):
                        if lo2.hold:
                            self.taken=None
                            self.takenCount=0
                        self.snow.remove(lo2)
                        self.levelObjects.remove(lo2)
                
                
                if self.taken<>None:
                    if self.taken.currentFrame==self.taken.snowFrame:
                        self.you.changeAnimation(self.you.frameReady)
                    
                if self.you.frameNbr==self.you.frameShot[1]:
                    self.opponent.changeAnimation(self.opponent.frameHit,1)


    def shot(self):
        self.you.changeAnimation(self.you.frameShot,1)
        self.taken=None
        self.takenCount=0
        if self.opponent.lives>0:
            self.opponent.lives-=1
            self.opponent.changeAnimation(self.opponent.frameHit,1)
        else:
            self.opponent.changeAnimation(self.opponent.frameHit3,1)         
                    

    
       
    def customKeysActions(self,keyEvent):
        if keyEvent.type == MOUSEMOTION:
            if self.taken==None:
                for lo in self.snow:
                    if lo.rect.collidepoint(pygame.mouse.get_pos()):
                        self.taken=lo
                        lo.hold=True
                        break
            else:
                self.takenCount+=1
                nextFrame=min(self.taken.snowFrame[0],self.taken.frameStand[0]+self.takenCount/40)
                self.taken.currentFrame=nextFrame,nextFrame

                


    
    def generateSnow(self):
        for lo in self.snow:
            lo.rect.left=lo.rect.left-lo.speed['left']+lo.speed['right']
            lo.rect.top=lo.rect.top-lo.speed['up']+lo.speed['down']        

        if self.levelTimer.counter%(2*FPS)==0:        
            gap=100
            left=random.randint(-gap,self.screen.get_width()+gap)
            if left<0 or left>self.screen.get_width():
                top=random.randint(-gap,self.screen.get_height()+gap)
            else:
                if random.randint(0,1)==0:
                    top=random.randint(-gap,0)
                else:
                    top=random.randint(self.screen.get_height(),self.screen.get_height()+gap)
            size=random.randrange(50,90,10)
            self.snow.append(self.addObject(objects.Snow(self,(size,size),(left,top))))  

