'''Game main module.

'''
import sys, pygame, random, math, os, time, traceback
import data
import objects, levels


from pygame.locals import *
pygame.init()
clock = pygame.time.Clock()


#fonts
CUSTOMFONT='Bloody.ttf'
HUGEFONT = pygame.font.Font(data.filepath(CUSTOMFONT), 60)
REGULARFONT = pygame.font.Font(data.filepath(CUSTOMFONT), 40)
TINYFONT = pygame.font.Font(data.filepath(CUSTOMFONT), 20)
#events
SCREEN_UPDATE_EVENT=pygame.USEREVENT+1
DEBUG_MODE=pygame.USEREVENT+2
TOGGLE_AUDIO=pygame.USEREVENT+3
FIRE=pygame.USEREVENT+4
#costants
FPS = 30
DEFAULT_SIZE=800, 600
PAUSESCREEN=-5
SPLASHSCREEN=-4
TITLESCREEN=-3
GAMEINTRO=-2
LEVELLIST=-1
PLANETSIZE=30
#level ID
SUN=0
MERCURY=1
VENUS=2
EARTH=3
MARS=4
JUPITER=5
SATURN=6
URANUS=7
NEPTUNE=8
PLUTO=9
#level status
BRIEFING=0
COUNTDOWN=1
STARTED=2
WIN=3
LOSE=4


#flags
debugOn=False
musicOn=True

#clock = pygame.time.Clock()
print '='*30
print 'Available resolutions:'
print 'FULLSCREEN'
avares=pygame.display.list_modes()
for lm in avares:
    if lm[0]>lm[1]:
        print str(lm[0])+' '+str(lm[1])
print 'Try different resolution by passing them as command line argument:'
print 'e.g. python run_game.py 1280 800'
    
try:
    if sys.argv[1]=='FULLSCREEN':
            screen = pygame.display.set_mode(DEFAULT_SIZE, pygame.FULLSCREEN)
    elif tuple([int(sys.argv[1]),int(sys.argv[2])]) in avares:
            screen = pygame.display.set_mode(tuple([int(sys.argv[1]),int(sys.argv[2])]))
    else:
        screen = pygame.display.set_mode(DEFAULT_SIZE)
except (IndexError, ValueError):
    screen = pygame.display.set_mode(DEFAULT_SIZE)
print 'Resolution set to:',screen.get_size()
print '='*30


#screen.blit(data.load_image('1_1_forked_tail_logo.png',(screen.get_width()*3/4,screen.get_height()*3/4)), (screen.get_width()/8,0))
#pygame.display.flip()


'''
Mercury 0.3 4.880 km
Venus 1 12.104 km
Earth 1 12.756 km
Mars 0.5 6.794 km
Jupiter 11 142.984 km
Saturn 10 120.536 km
Uranus 4 51.118 km
Neptune 4 49.532 km
Sun 100 1.390.000 km
Pluto 0.2 2.274 km
'''

animDict={}
animDict['forked_tail_logo']=screen.get_width()*3/4,screen.get_height()*3/4
animDict['title']=screen.get_height(),screen.get_height()

myAnims={}
myAnims['forked_tail_logo']=data.load_animation('1_1_forked_tail_logo.png', (screen.get_width(),screen.get_height()))
#myAnims=data.load_all_animations(animDict)
data.load_music('PT9P_theme.mp3')
#mySounds=data.load_all_sounds()
levelsCompleted={SUN:False,MERCURY:False,VENUS:False,EARTH:False,MARS:False,JUPITER:False,SATURN:False,URANUS:False,NEPTUNE:False}
#print 'Press a key to continue'

def main():
    PlutoLives=9
    currentLevel=freezedLevel=None
    debugOn=False
    musicOn=True
    levelNum=SPLASHSCREEN
    #levelNum=MERCURY
    FATALERROR=False    
    while not FATALERROR:
        if levelNum<>SPLASHSCREEN and not pygame.mixer.music.get_busy() and musicOn:
            pygame.mixer.music.play(-1)
        if freezedLevel<>None:
            currentLevel=freezedLevel
            freezedLevel=None
            currentLevel.levelFreezed=False
            currentLevel.screen.blit(currentLevel.backgroundImage, (0,0))
            levelNum=currentLevel.num
        else:
            if currentLevel<>None and currentLevel.levelFreezed==True:
                freezedLevel=currentLevel
            if levelNum==PAUSESCREEN:
                currentLevel=levels.PauseScreen(levelNum,screen,myAnims,debugOn,musicOn)
            elif levelNum==SPLASHSCREEN:
                currentLevel=levels.Splash(levelNum,screen,myAnims,debugOn,musicOn)
            elif levelNum==TITLESCREEN:
                currentLevel=levels.Title(levelNum,screen,myAnims,debugOn,musicOn)
            elif levelNum==GAMEINTRO:
                currentLevel=levels.GameIntro(levelNum,screen,myAnims,debugOn,musicOn)                
            elif levelNum==LEVELLIST:
                currentLevel=levels.LevelList(levelNum,screen,myAnims,debugOn,musicOn,levelsCompleted,PlutoLives)   
            elif levelNum==MERCURY:
                currentLevel=levels.MercuryLevel(levelNum,screen,myAnims,debugOn,musicOn) 
            elif levelNum==VENUS:
                currentLevel=levels.VenusLevel(levelNum,screen,myAnims,debugOn,musicOn) 
            elif levelNum==EARTH:
                currentLevel=levels.EarthLevel(levelNum,screen,myAnims,debugOn,musicOn)  
            elif levelNum==MARS:
                currentLevel=levels.MarsLevel(levelNum,screen,myAnims,debugOn,musicOn)      
            elif levelNum==JUPITER:
                currentLevel=levels.JupiterLevel(levelNum,screen,myAnims,debugOn,musicOn)                              
            elif levelNum==URANUS:
                currentLevel=levels.UranusLevel(levelNum,screen,myAnims,debugOn,musicOn)    
            elif levelNum==SATURN:
                currentLevel=levels.SaturnLevel(levelNum,screen,myAnims,debugOn,musicOn)                                
            elif levelNum==NEPTUNE:
                currentLevel=levels.NeptuneLevel(levelNum,screen,myAnims,debugOn,musicOn) 
            else:
                print 'WARNING'
                currentLevel=levels.LevelList(levelNum,screen,myAnims,debugOn,musicOn,levelsCompleted,PlutoLives)               

        while currentLevel.levelComplete==False and currentLevel.levelFreezed==False:
            try:
                for event in pygame.event.get():
                    levelNum,debugOn,musicOn=currentLevel.eventsManagement(event)                
                
                levelNum,debugOn,musicOn=currentLevel.update()
                clock.tick(FPS) 
                pygame.display.flip()
            except:
                raise
                FATALERROR=True
                errorfile=open('errors.log', 'a')
                errorfile.write('-----FATAL ERRROR-----'+'\n'+str(time.localtime())+'\n')
                traceback.print_exc(file=errorfile)
                print '-------------------------------------\nFATAL ERRROR. Please check errors.log\n-------------------------------------'
                break
        if currentLevel.levelComplete==True and currentLevel.levelFreezed==False:
            if currentLevel.status==WIN:
                levelsCompleted[currentLevel.num]=True
            elif currentLevel.status==LOSE:
                PlutoLives-=1
        if PlutoLives<=0:
            print 'YOU LOSE'
            sys.exit()
        

