import os, pygame, random, math
import data
from pygame.locals import *
pygame.init()

#fonts
CUSTOMFONT='Bloody.ttf'
HUGEFONT = pygame.font.Font(data.filepath(CUSTOMFONT), 60)
REGULARFONT = pygame.font.Font(data.filepath(CUSTOMFONT), 40)
TINYFONT = pygame.font.Font(data.filepath(CUSTOMFONT), 20)
#events
SCREEN_UPDATE_EVENT=pygame.USEREVENT+1
DEBUG_MODE=pygame.USEREVENT+2
TOGGLE_AUDIO=pygame.USEREVENT+3
FIRE=pygame.USEREVENT+4
#costants
FPS = 30
DEFAULT_SIZE=800, 600
PAUSESCREEN=-5
SPLASHSCREEN=-4
TITLESCREEN=-3
GAMEINTRO=-2
LEVELLIST=-1
PLANETSIZE=30
#level ID
SUN=0
MERCURY=1
VENUS=2
EARTH=3
MARS=4
JUPITER=5
SATURN=6
URANUS=7
NEPTUNE=8
PLUTO=9

ANITIME=15




class LevelMask(pygame.sprite.Sprite):  
    def __init__(self, size, topleft, color=(0,0,0)):
        pygame.sprite.Sprite.__init__(self)
        self.initialtopleft=topleft
        self.color=color
        self.image=pygame.Surface((size))
        self.image.fill(color)
        self.rect=self.image.get_rect()
        self.rect.topleft=topleft
    
    def update(self):
        self.image=pygame.Surface((self.rect.size))
        self.image.fill(self.color)
    


class GameText(pygame.sprite.Sprite):  
    def __init__(self, text, size, topleft, color=pygame.Color('DeepSkyBlue')):
        pygame.sprite.Sprite.__init__(self)
        self.changed=False
        self.text=text
        self.size=size
        self.initialtopleft=topleft
        self.color=color
        self.font=pygame.font.Font(data.filepath(CUSTOMFONT), self.size)
        self.image=self.font.render(self.text, True, self.color)
        self.rect=self.image.get_rect()
        self.rect.topleft=topleft
    
    def update(self):
        if self.changed:
            self.font=pygame.font.Font(data.filepath(CUSTOMFONT), self.size)
            self.image=self.font.render(self.text, True, self.color)            
            self.changed=False

    def editText(self,newtext):
        self.text=str(newtext)
        self.changed=True
        
    def moveText(self,newtopleft):
        self.topleft=newtopleft
        self.changed=True
        
    def resizeText(self,newsize):
        self.size=newsize
        self.changed=True
        
    def colorText(self,newcolor):
        self.color=color
        self.changed=True


class LevelTimer(GameText):
    def __init__(self, size, topleft, color=(255,255,255)):
        GameText.__init__(self, '', size, topleft, color=(255,255,255))
        self.counter=0
        self.show=''
    
    def update(self):
        self.counter+=1
        if self.show:
            self.editText(str((self.counter/FPS)/60).rjust(2,'0')+':'+str((self.counter/FPS)%60).rjust(2,'0')+':'+str(100*(self.counter%30+1)/30).rjust(2,'0'))
        else:
            self.editText('')
        self.font=pygame.font.Font(data.filepath(CUSTOMFONT), self.size)
        self.image=self.font.render(self.text, True, self.color)            


class SpaceObject(pygame.sprite.Sprite):  
    def __init__(self, ID, name, level, size, topleft):
        pygame.sprite.Sprite.__init__(self)
        self.ID=ID
        self.name=name
        self.level=level
        self.size=size
        self.frameNbr=0
        self.animationTimer=0        
        self.animations=[]
        for a in level.myAnims[self.name]:
            self.animations.append(pygame.transform.scale(a,self.size))
        if len(self.animations)>0:
            self.image=self.animations[self.frameNbr]
            self.rect=self.animations[self.frameNbr].get_rect()
            self.rect.topleft=topleft
            self.radius=self.rect.width/4 
        
        self.speed=0.0 
        self.speed={'left':0.0,'right':0.0,'up':0.0,'down':0.0} 
        self.engine={'left':0,'right':0,'up':0,'down':0} 
        self.direction='left'
        self.mass=0.0
        self.gravity=0.0
        self.orbitRadius=self.rect.left-self.level.sscenter[0]
        self.orbitSpeed=0.0
        self.greenery=0
        self.win=False
        self.lose=False        
        self.lives=1
        
        '''
        self.frameStand=0,0
        self.frameWin=0,0
        self.frameLose=0,0
        self.currentFrame=self.frameStand
        '''


        '''
        Earth 12.756 km
        Venus 1 12.104 km
        Mars 0.5 6.794 km
        Mercury 0.3 4.880 km
        Pluto 0.2 2.274 km
        Jupiter 11 142.984 km
        Saturn 10 120.536 km
        Uranus 4 51.118 km
        Neptune 4 49.532 km
        Sun 100 1.390.000 km
        '''
        
        
       

    def update(self):
        self.customUpdate()
        self.animationTimer=(self.animationTimer+1)%ANITIME
        if self.win:
            self.currentFrame=self.frameWin
        elif self.lose:
            self.currentFrame=self.frameLose        
        if self.animationTimer==0:
            self.updateAnimation(self.currentFrame)

        #self.rect.left=self.rect.left+self.a*math.cos(math.pi*self.level.t)
        #self.rect.top=self.rect.top+self.b*math.sin(math.pi*self.level.t)
        #print math.pi*self.level.t,self.rect.left,self.a*math.cos(math.pi*self.level.t)
            
    
    def customUpdate(self):
        pass

    def updateAnimation(self,frames,step=1):
        start,end=frames
        delta=end+1-start
        if self.frameNbr<start or self.frameNbr>end:
            self.frameNbr=start-1
        self.frameNbr=start+(self.frameNbr-start+step)%delta
        self.image=self.animations[self.frameNbr]
        if self.direction=='right':
            self.image=pygame.transform.flip(self.image, True, False)    

class Planet(SpaceObject):  
    def __init__(self, ID, name, level, size, topleft, levelCompleted):
        SpaceObject.__init__(self, ID, name, level, size, topleft)
        pygame.sprite.Sprite.__init__(self)
        self.ID=ID
        self.name=name
        self.level=level
        self.levelCompleted=levelCompleted
        self.size=size
        self.frameNbr=0
        self.animationTimer=0        
        self.animations=[]
        for a in level.myAnims[self.name]:
            self.animations.append(pygame.transform.scale(a,self.size))
        if len(self.animations)>0:
            self.image=self.animations[self.frameNbr]
            self.rect=self.animations[self.frameNbr].get_rect()
            self.rect.topleft=topleft
            self.radius=self.rect.width/4 
        
        self.speed=0.0 
        self.speed={'left':0.0,'right':0.0,'up':0.0,'down':0.0} 
        self.engine={'left':0,'right':0,'up':0,'down':0} 
        self.direction='left'
        self.mass=0.0
        self.gravity=0.0
        self.orbitRadius=self.rect.left-self.level.sscenter[0]
        self.orbitSpeed=0.0
        self.greenery=0
        self.win=False
        self.lose=False        
        self.lives=1
        

        
        self.animationSetup()
        self.currentFrameLoop=[self.frameDefault,-1]
        self.nextFrameLoop=[self.frameDefault,-1]
        if levelCompleted:
            self.currentFrameLoop=[self.frameLevelComplete,-1]
            self.nextFrameLoop=[self.frameLevelComplete,-1]


        '''
        Earth 12.756 km
        Venus 1 12.104 km
        Mars 0.5 6.794 km
        Mercury 0.3 4.880 km
        Pluto 0.2 2.274 km
        Jupiter 11 142.984 km
        Saturn 10 120.536 km
        Uranus 4 51.118 km
        Neptune 4 49.532 km
        Sun 100 1.390.000 km
        '''
    '''    
    def animationSetup(self):
        self.frameDefault=0,0
        self.frameLevelComplete=0,0
        self.frameStand=0,0
        self.frameWin=0,0
        self.frameLose=0,0  
    '''

    def update(self):
        self.customUpdate()
        self.animationTimer=(self.animationTimer+1)%ANITIME
        if self.win:
            self.changeAnimation(self.frameWin)
        elif self.lose:
            self.changeAnimation(self.frameLose)       
        if self.animationTimer==0:
            self.updateAnimationNEW()

        #self.rect.left=self.rect.left+self.a*math.cos(math.pi*self.level.t)
        #self.rect.top=self.rect.top+self.b*math.sin(math.pi*self.level.t)
        #print math.pi*self.level.t,self.rect.left,self.a*math.cos(math.pi*self.level.t)
            
    
    def customUpdate(self):
        pass

    def updateAnimationOld(self,frames,step=1):
        start,end=frames
        delta=end+1-start
        if self.frameNbr<start or self.frameNbr>end:
            self.frameNbr=start-1
        self.frameNbr=start+(self.frameNbr-start+step)%delta
        self.image=self.animations[self.frameNbr]
        if self.direction=='right':
            self.image=pygame.transform.flip(self.image, True, False)
            
    def changeAnimation(self,newAnimation,loop=-1,step=1):
        '''
        loop=0 means loop forever
        '''
        if loop>0:
            self.nextFrameLoop=self.currentFrameLoop
        else:
            self.nextFrameLoop=[newAnimation,loop]
        self.currentFrameLoop=[newAnimation,loop]

    
    def updateAnimationNEW(self,step=1):
        startend,loop=self.currentFrameLoop
        start,end=startend
        delta=end+1-start
        if self.frameNbr<start or self.frameNbr>end:
            self.frameNbr=start-1
        elif self.frameNbr==end:
            if loop>0:
                #restart cicle
                self.currentFrameLoop[1]-=1
            elif loop==0:
                #cicle end, restore default animation
                self.changeAnimation(self.frameDefault)
                return
            self.frameNbr=start-1
        self.frameNbr=start+(self.frameNbr-start+step)%delta
        self.image=self.animations[self.frameNbr]


class Sun(Planet):  
    def __init__(self, level, size, topleft, levelCompleted=False):
        self.ID=SUN
        self.name='sun'
        Planet.__init__(self, self.ID, self.name, level, size, topleft, levelCompleted)
        self.mass=10000 #10^22 kg
        self.gravity=1000 # m/s2
        self.orbitSpeed=0.0
        
    def animationSetup(self):
        self.frameDefault=9,9
        self.frameLevelComplete=10,10
        self.frameStand=9,9
        self.frameWin=9,9
        self.frameLose=10,10
        

    def customUpdate(self):
        pass


        

class Mercury(Planet):  
    def __init__(self, level, size, topleft, levelCompleted=False):
        self.ID=MERCURY
        self.name='mercury'
        Planet.__init__(self, self.ID, self.name, level, size, topleft, levelCompleted)
        self.win=False
        self.lose=False

        self.orbitSpeed=0.5



    def animationSetup(self):
        self.frameDefault=0,0
        self.frameLevelComplete=9,10
        self.frameStand=13,13
        self.frameStart=0,0
        self.frameWalk=1,4  
        self.frameRun=5,7
        self.frameJump=8,8
        self.frameWin=11,12
        self.frameLose=9,10
         
        
    def customUpdate(self):
        #to be fixed
        if self.level.num==MERCURY:
            if self.speed['left']==0.0:
                self.changeAnimation(self.frameStart)
            elif self.speed['left']<=2.0:
                self.changeAnimation(self.frameWalk)
            elif self.speed['left']>2.0:
                self.changeAnimation(self.frameRun)

class Venus(Planet):  
    def __init__(self, level, size, topleft, levelCompleted=False):
        self.ID=VENUS
        self.name='venus'
        Planet.__init__(self, self.ID, self.name, level, size, topleft, levelCompleted)
        self.mass=480 #10^22 kg
        self.gravity=9 # m/s2

        self.orbitSpeed=0.4
        
    def animationSetup(self):
        self.frameDefault=0,0
        self.frameLevelComplete=1,1             
        self.frameStand=0,0
        self.frameCollect=1,1
        self.frameWin=1,1
        self.frameLose=1,1

    def customUpdate(self):
        pass
        
class Earth(Planet):  
    def __init__(self, level, size, topleft, levelCompleted=False):
        self.ID=EARTH
        self.name='earth'
        Planet.__init__(self, self.ID, self.name, level, size, topleft, levelCompleted)
        self.mass=480 #10^22 kg
        self.gravity=9 # m/s2
        self.greenery=75
        
        self.orbitSpeed=0.3
    
    def animationSetup(self):
        self.frameDefault=0,0
        self.frameLevelComplete=5,5
        self.frameGreenery=0,4
        self.frameWin=0,0
        self.frameLose=5,5

    def customUpdate(self):
        pass  
        
class Moon(Planet):  
    def __init__(self, level, size, topleft, levelCompleted=False):
        self.ID=0
        self.name='moon'
        Planet.__init__(self, self.ID, self.name, level, size, topleft, levelCompleted)
        self.mass=480 #10^22 kg
        self.gravity=9 # m/s2
        self.greenery=75
        
        self.orbitSpeed=0.3
    
    def animationSetup(self):
        self.frameDefault=0,0
        self.frameStand=0,0
        self.frameFold=1,4
        self.frameNoSign=3,5
        self.frameUnfold=4,7
        self.frameEasterEgg=8,8
        

    def customUpdate(self):
        pass          
        
class Mars(Planet):  
    def __init__(self, level, size, topleft, levelCompleted=False):
        self.ID=MARS
        self.name='mars'
        Planet.__init__(self, self.ID, self.name, level, size, topleft, levelCompleted)
        self.mass=480 #10^22 kg
        self.gravity=9 # m/s2
        self.lives=3
        
        self.orbitSpeed=0.2
        
    def animationSetup(self):
        self.frameDefault=1,2
        self.frameLevelComplete=6,6             
        self.frameStand=0,0
        self.frameDefence=1,2
        self.frameAttack=3,5
        self.frameHit=6,6
        self.frameLose=7,9
        self.frameWin=10,11

    def customUpdate(self):
        pass
            
        
class Jupiter(Planet):  
    def __init__(self, level, size, topleft, levelCompleted=False):
        self.ID=JUPITER
        self.name='jupiter'
        Planet.__init__(self, self.ID, self.name, level, size, topleft, levelCompleted)
        self.mass=480 #10^22 kg
        self.gravity=9 # m/s2
        self.lives=1
        self.orbitSpeed=0.5

    def animationSetup(self):
        self.frameDefault=0,0
        self.frameLevelComplete=0,0             
        self.frameStand=0,0
        self.frameBlow1=1,3
        self.frameKeep1=4,5
        self.frameBlow2=7,9
        self.frameKeep2=10,11
        self.frameExplode=13,15
        self.frameDeflate=16,18
        self.frameWin=12,12
        self.frameLose=0,0

    def customUpdate(self):
        pass                           
        
class Saturn(Planet):  
    def __init__(self, level, size, topleft, levelCompleted=False):
        self.ID=SATURN
        self.name='saturn'
        Planet.__init__(self, self.ID, self.name, level, size, topleft, levelCompleted)
        self.mass=56000 #10^22 kg
        self.gravity=9 # m/s2
        self.orbitSpeed=0.4
        
    def animationSetup(self):
        self.frameDefault=0,0
        self.frameLevelComplete=12,12             
        self.frameStand=0,0
        self.frameWaiting=1,11
        self.frameWin=1,11
        self.frameLose=12,12

    def customUpdate(self):
        pass  
        
class Uranus(Planet):  
    def __init__(self, level, size, topleft, levelCompleted=False):
        self.ID=URANUS
        self.name='uranus'
        Planet.__init__(self, self.ID, self.name, level, size, topleft, levelCompleted)
        self.mass=480 #10^22 kg
        self.gravity=9 # m/s2
        self.orbitSpeed=0.3
        self.lives=3
        self.found=False
        

    def animationSetup(self):
        self.frameDefault=0,0
        self.frameLevelComplete=6,6             
        self.frameStand=0,0
        self.frameFound=0,3
        self.frameFoundAgain=0,6
        self.frameWin=7,8        
        self.frameLose=6,6

        

    def customUpdate(self):
        pass
                

class Neptune(Planet):  
    def __init__(self, level, size, topleft, levelCompleted=False):
        self.ID=NEPTUNE
        self.name='neptune'
        Planet.__init__(self, self.ID, self.name, level, size, topleft, levelCompleted)
        self.frameLose=1,1
        self.mass=10200 #10^22 kg
        self.gravity=11 # m/s2
        self.orbitSpeed=0.2
        self.lives=3

    def animationSetup(self):
        self.frameDefault=0,0
        self.frameLevelComplete=0,0             
        self.frameStand=0,0
        self.frameHit=2,11
        self.frameHit3=2,16
        self.frameWin=14,15
        self.frameLose=16,16

    def customUpdate(self):
        pass
        
        
 
class Pluto(Planet):  
    def __init__(self, level, size, topleft, levelCompleted=False):
        self.ID=PLUTO
        self.name='pluto'
        Planet.__init__(self, self.ID, self.name, level, size, topleft, levelCompleted)
        self.win=False
        self.lose=False
        self.mass=1.3 #*10^22 kg
        self.gravity=0.6 #m/s2
        self.greenery=25
        self.currentFrame=self.frameStand
    

    def animationSetup(self):
        self.frameDefault=0,0
        self.frameStand=0,0
        self.frameWin=1,2
        self.frameLose=3,4
        
        if self.level.num==MERCURY:
            self.frameDefault=5,5
            self.frameStand=5,5
            self.frameStart=5,5
            self.frameWalk=6,7  
            self.frameRun=6,7
            self.frameJump=0,0
        
        if self.level.num==VENUS:
            self.frameCollect=1,2 

        if self.level.num==EARTH:
            self.frameGreenery=9,13
            
        if self.level.num==MARS:
            self.lives=3
            self.frameDefault=14,15
            self.frameStand=14,15
            self.frameDefence=14,15
            self.frameAttack=14,17
            self.frameAttackFail=17,17
            self.frameHit=18,18
            self.frameLose=18,18
            self.frameWin=14,14

        if self.level.num==JUPITER:
            self.lives=1
            self.frameBlow=20,22
            self.frameKeep=23,23
            self.frameExplode=24,26

        if self.level.num==SATURN:
            self.frameDefault=27,27
            self.frameStand=27,27
        
        if self.level.num==URANUS:
            self.frameDefault=38,38
            self.frameStand=38,38
            self.frameFound=39,39

        if self.level.num==NEPTUNE:
            self.frameDefault=28,29
            self.frameStand=28,29
            self.frameReady=30,30
            self.frameShot=30,33

        

        
    def customUpdate(self):
        ###
        if self.level.num==MERCURY:
            #self.speed=max(self.speed-0.1,0)
            if self.speed['left']==0.0:
                self.changeAnimation(self.frameStand)
            elif self.speed['left']<=2.0:
                self.changeAnimation(self.frameWalk)
            elif self.speed['left']>2.0:
                self.changeAnimation(self.frameRun)
        ###
        elif self.level.num==VENUS:
            for k in self.engine.keys():
                self.speed[k]=self.speed[k]+self.engine[k]*0.1
            if False:
                self.changeAnimation(self.frameCollect)
            else:
                self.changeAnimation(self.frameStand)
        '''
        elif self.level.num==EARTH:
            self.changeAnimation(self.frameStand)
        elif self.level.num==MARS:
            self.changeAnimation(self.frameStand)   
        elif self.level.num==JUPITER:
            self.changeAnimation(self.frameStand) 
        elif self.level.num==URANUS:
            self.changeAnimation(self.frameStand)
        elif self.level.num==NEPTUNE:
            self.changeAnimation(self.frameStand)
        else:
            print 'error on pluto update'
        '''
            

 
                            
class Asteroid(SpaceObject):  
    def __init__(self, level, size, topleft):
        self.ID=0
        self.name='asteroid'
        SpaceObject.__init__(self, self.ID, self.name, level, size, topleft)
        randFrame=random.randint(0,7)
        self.frameStand=randFrame,randFrame
        self.currentFrame=self.frameStand
        self.mass=round(random.random(),1)*100
        self.greenery=-5
        
        #self.speed={'left':round(random.random(),1),'right':round(random.random(),1),'up':round(random.random(),1),'down':round(random.random(),1)}
        if self.rect.left<self.level.screen.get_rect().centerx:
            self.speed['right']=random.randint(1,3)
        else:
            self.speed['left']=random.randint(1,3)
        if self.rect.top<self.level.screen.get_rect().centery:
            self.speed['down']=random.randint(1,3)
        else:
            self.speed['up']=random.randint(1,3) 
    
    def customUpdate(self):
        pass
                
        
class Seed(SpaceObject):  
    def __init__(self, level, size, topleft):
        self.ID=0
        self.name='seed'
        SpaceObject.__init__(self, self.ID, self.name, level, size, topleft)
        randFrame=random.randint(0,7)
        self.frameStand=randFrame,randFrame
        self.currentFrame=self.frameStand
        self.mass=0
        self.greenery=5

        if self.rect.left<self.level.screen.get_rect().centerx:
            self.speed['right']=random.randint(1,3)
        else:
            self.speed['left']=random.randint(1,3)
        if self.rect.top<self.level.screen.get_rect().centery:
            self.speed['down']=random.randint(1,3)
        else:
            self.speed['up']=random.randint(1,3) 
    
    def customUpdate(self):
        pass        


class Snow(SpaceObject):  
    def __init__(self, level, size, topleft):
        self.ID=0
        self.name='snow'
        SpaceObject.__init__(self, self.ID, self.name, level, size, topleft)
        self.frameStand=0,0
        self.currentFrame=self.frameStand
        self.snowFrame=5,5
        self.mass=0
        self.hold=False

        if self.rect.left<self.level.screen.get_rect().centerx:
            self.speed['right']=random.randint(1,3)
        else:
            self.speed['left']=random.randint(1,3)
        if self.rect.top<self.level.screen.get_rect().centery:
            self.speed['down']=random.randint(1,3)
        else:
            self.speed['up']=random.randint(1,3) 
    
    def customUpdate(self):
        pass    